package com.framsticks.communication;

import com.framsticks.communication.queries.*;

import java.util.Collection;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Class stores information about query sent to manager.
 */
public abstract class Request {

    public static void quoteValue(StringBuilder builder, String value) {
		String quote = ((value.indexOf(' ') > 0) || (value.length() == 0) ? "\"" : "");
		builder.append(quote).append(value).append(quote);
	}

	public abstract String getCommand();

    protected abstract StringBuilder construct(StringBuilder buffer);
	//private static Pattern queryPattern = Pattern.compile("^(\\S+)\\s+(\\S+)(?:\\s+(\\S+))?$");

	public static Request createRequestByTypeString(String type) {
		if (type.equals("get")) {
			return new GetRequest();
		}
		if (type.equals("set")) {
			return new SetRequest();
		}
		if (type.equals("info")) {
			return new InfoRequest();
		}
		if (type.equals("call")) {
			return new CallRequest();
		}
        if (type.equals("reg")) {
            return new RegistrationRequest();
        }
        if (type.equals("use")) {
            return new UseRequest();
        }
        if (type.equals("version")) {
            return new VersionRequest();
        }
        return null;
	}

    public abstract void parseRest(String rest);

    @Override
    public String toString() {
        return construct(new StringBuilder().append(getCommand())).toString();
    }
}