package com.framsticks.communication;

import com.framsticks.communication.queries.*;
import com.framsticks.params.SourceInterface;
import com.framsticks.util.Pair;
import com.framsticks.util.Strings;
import org.apache.log4j.Logger;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;

/**
 * @author Piotr Sniegowski
 */
public class ServerConnection extends Connection {

    private final static Logger LOGGER = Logger.getLogger(ServerConnection.class);

    RequestHandler requestHandler;

    public ServerConnection(Socket socket, RequestHandler requestHandler) {
        this.socket = socket;
        this.requestHandler = requestHandler;
        connected = true;

    }

    public void start() {


        runThreads();
    }

    @Override
    public String toString() {
        return socket.getInetAddress().getHostAddress();
    }

    @Override
    protected void receiverThreadRoutine() throws Exception {
        while (connected) {
            processNextRequest();
        }
    }

    protected void handleRequest(Request request, ResponseCallback responseCallback) {
        if (request instanceof ApplicationRequest) {
            requestHandler.handle((ApplicationRequest)request, responseCallback);
            return;
        }
        if (request instanceof ProtocolRequest) {
            if (request instanceof VersionRequest) {
                responseCallback.process(new Response(true, null, null));
                return;
            }
            if (request instanceof UseRequest) {
                String feature = ((UseRequest)request).getFeature();
                if (feature.equals("request_id")) {
                    requestIdEnabled = true;
                    responseCallback.process(new Response(true, null, null));
                    return;
                }
                responseCallback.process(new Response(false, "\"unknown feature: " + feature + "\"", null));
                return;
            }

        }
        LOGGER.error("unhandled request: " + request);
        responseCallback.process(new Response(false, "unhandled", null));
    }

    protected final void respond(final Response response, final Integer id) {
        senderThread.invokeLater(new Runnable() {
            @Override
            public void run() {
                String outId = id != null ? " " + id : "";
                if (response.getFiles() != null) {
                    for (File f : response.getFiles()) {
                        output.print("file" + outId/* + " " + f.getPath()*/ + "\n");
                        SourceInterface content = f.getContent();
                        String line;
                        while ((line = content.readLine()) != null) {
                            output.print(line);
                            output.print('\n');
                        }
                        output.print("eof\n");
                    }
                }
                output.print(response.getOk() ? "ok" : "error");
                output.print(outId);
                if (Strings.notEmpty(response.getComment())) {
                    output.print(' ');
                    output.print(response.getComment());
                }
                output.print('\n');
                output.flush();
            }
        });

    }

    protected void processNextRequest() throws Exception {
        String line = getLine();
        Pair<String, String> command = Strings.splitIntoPair(line, ' ', "\n");
        final Pair<Integer, String> rest = parseRest(command.second);
        if (rest == null) {
            respond(new Response(false, "\"invalid input\"", null), null);
            return;
        }

        final Request request = Request.createRequestByTypeString(command.first);
        if (request == null) {
            respond(new Response(false, "\"invalid input\"", null), null);
            return;
        }
        request.parseRest(rest.second);
        if (LOGGER.isTraceEnabled()) {
            LOGGER.trace("read request: " + request);
        }

        handleRequest(request, new ResponseCallback() {
            @Override
            public void process(Response response) {
                respond(response, rest.first);
            }
        });
    }
}
