package com.framsticks.communication;

import com.framsticks.communication.queries.RegistrationRequest;
import com.framsticks.util.AtOnceDispatcher;
import com.framsticks.util.Dispatcher;
import com.framsticks.util.StateFunctor;
import org.apache.log4j.Logger;


/**
 * @author Piotr Sniegowski
 */
public class Subscription {

    private final static Logger LOGGER = Logger.getLogger(Subscription.class);

    private final ClientConnection connection;
    private final String path;
	private final String registeredPath;
    private Dispatcher dispatcher = AtOnceDispatcher.instance;

	private EventCallback eventCallback;

	public Subscription(ClientConnection connection, String path, String registeredPath) {
		this.connection = connection;
        this.path = path;
		this.registeredPath = registeredPath;
	}

	public String getPath () {
		return path;
	}

	public String getRegisteredPath() {
		return registeredPath;
	}

	@Override
	public String toString() {
		return path + "(" + registeredPath + ")";
	}

    public void setDispatcher(Dispatcher dispatcher) {
        this.dispatcher = dispatcher;
    }

    public void unsubscribe(final StateFunctor stateFunctor) {
        //@todo remove that /cli/ prefix, when registeredPath will be a fully qualified path
        connection.send(new RegistrationRequest().register(false).setPath(registeredPath), new ResponseCallback() {
            @Override
            public void process(Response response) {
                if (!response.getOk()) {
                    LOGGER.error("failed to unsunscribe " + this + ": " + response.getComment());
                    stateFunctor.call(new Exception(response.getComment()));
                    return;
                }
                assert response.hasFiles();
                LOGGER.debug("unsunscribed " + this);
                stateFunctor.call(null);
            }
        });
    }

    public EventCallback getEventCallback() {
        return eventCallback;
    }

    public Dispatcher getDispatcher() {
        return dispatcher;
    }

    public void setEventCallback(EventCallback eventCallback) {
        this.eventCallback = eventCallback;
    }
}
