package com.framsticks.dumping;

import com.framsticks.communication.File;
import com.framsticks.core.Path;
import com.framsticks.params.ListSource;
import com.framsticks.core.Instance;
import com.framsticks.util.*;
import org.apache.log4j.Logger;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

/**
 * @author Piotr Sniegowski
 */
public class LoadStream {

    private final static Logger LOGGER = Logger.getLogger(LoadStream.class.getName());

    protected final Instance instance;
    protected final Path mountPath;
    protected final BufferedReader stream;
    protected final Future<Path> future;
    protected final Stopwatch stopwatch = new Stopwatch();


    public LoadStream(Path mountPath, BufferedReader stream, Instance instance, Future<Path> future) {
        this.instance = instance;
        this.mountPath = mountPath;
        this.stream = stream;
        this.future = future;
    }

    public void load() {
        try {
            String line;
            Pair<String, String> query = null;
            List<File> files = null;
            List<String> content = null;
            //File file;
            while ((line = stream.readLine()) != null) {
                if (query == null) {
                    query = Strings.splitIntoPair(line, ' ', "\n");
                    files = new LinkedList<File>();
                    LOGGER.trace("loading " + line);
                    continue;
                }
                if (content == null) {
                    if (line.equals("file")) {
                        content = new LinkedList<String>();
                        continue;
                    }
                    if (line.equals("ok")) {
                        if (query.first.equals("get")) {
                            Path path = instance.createIfNeeded(query.second);
                            instance.processFetchedValues(path, files);
                        } else if (query.first.equals("info")) {
                            assert files.size() == 1;
                            instance.processFetchedInfo(files.get(0));
                        } else {
                            assert false;
                        }
                        query = null;
                        files = null;
                        continue;
                    }
                    assert false;
                    continue;
                }
                if (line.equals("eof")) {
                    files.add(new File(query.second, new ListSource(content)));
                    content = null;
                    continue;
                }
                content.add(line);
            }
        } catch (IOException e) {
            LOGGER.error("failed to load: " + e);
            future.result(null, e);
            return;
        }
        LOGGER.info("loaded in: " + stopwatch);
        future.result(new Path(instance, mountPath.getTextual()), null);
    }


}
