package com.framsticks.gui;

import com.framsticks.params.AccessInterface;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * @author Piotr Sniegowski
 */
public abstract class Panel extends JPanel {

	private static final Logger LOGGER = Logger.getLogger(Panel.class.getName());

	/**
	 * Pane to which components will be added.
	 */
	protected JPanel contentPanel;

	protected TreeNode currentTreeNode;
    protected EndpointAtFrame endpoint;
    protected final Frame frame;
	protected final String className;
	final protected JLabel label;
	final protected JButton applyButton;

	public Panel(EndpointAtFrame endpoint, String className) {
        this.endpoint = endpoint;
		this.frame = endpoint.getFrame();
		this.className = className;
		LOGGER.debug("create panel for type: " + className);

        endpoint.registerPanel(this);

		contentPanel = new JPanel();
		contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.PAGE_AXIS));
		JScrollPane scroll = new JScrollPane(contentPanel);
		//scroll.setBorder(BorderFactory.createEtchedBorder());

		JPanel pageEndPanel = new JPanel();
		pageEndPanel.setLayout(new BoxLayout(pageEndPanel, BoxLayout.X_AXIS));
		pageEndPanel.add(Box.createHorizontalGlue());

		applyButton = new JButton("Apply");
		applyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				applyChanges();
			}
		});
		pageEndPanel.add(applyButton);

		pageEndPanel.add(Box.createHorizontalStrut(10));
		pageEndPanel.setPreferredSize(new Dimension(0, 30));

		label = new JLabel();
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new BorderLayout());
		centerPanel.add(Box.createHorizontalStrut(10), BorderLayout.LINE_START);
		centerPanel.add(Box.createHorizontalStrut(10), BorderLayout.LINE_END);
		centerPanel.add(label, BorderLayout.PAGE_START);
		centerPanel.add(scroll, BorderLayout.CENTER);
		centerPanel.add(pageEndPanel, BorderLayout.PAGE_END);

		this.setLayout(new BorderLayout());
		this.add(centerPanel, BorderLayout.CENTER);


	}

	public void setCurrentTreeNode(TreeNode node) {
		currentTreeNode = node;
		//label.setText(node.getNode().getPath());
		//this.revalidate();
	}

	public TreeNode getCurrentTreeNode() {
		return currentTreeNode;
	}

	protected abstract void applyChanges();
	public abstract void refreshComponents(AccessInterface access);

	@Override
	public String toString() {
		return currentTreeNode == null ? "?" : currentTreeNode.toString();
	}

    public final Frame getFrame() {
        return frame;
    }


    public final String getClassName() {
        return className;
    }

    public final String getFullName() {
        return className + "@" + endpoint.getName();
    }

}
