package com.framsticks.gui.components;

import com.framsticks.params.Flags;
import com.framsticks.params.Param;
import com.framsticks.params.types.*;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;

/**
 * Interface that each class that want to be component must implement.
 */
public abstract class Control extends JPanel {

	public static final int LINE_HEIGHT = 36;

	private static final Logger LOGGER = Logger.getLogger(Control.class.getName());

    protected Frame frame;
	protected Param param;

	public Control(Param param) {
		this.param = param;

		String help = param.getHelp();
		if (help != null) {
			this.setToolTipText(help);
		}

		if (param.getHelp() != null) {
			this.setToolTipText(param.getHelp());
		}
		this.setEnabled(!param.hasFlag(Flags.READONLY));
		// TODO: take care of textField exception: setEditable
	}

	public Param getParam() {
		return param;
	}

    public void setFrame(Frame frame) {
        this.frame = frame;
    }

	public static Control createComponentForText(ValueParam valueParam) {
		if (valueParam.getMin(Object.class) != null) {
			return new TextAreaControl(valueParam);
		}
		return new TextFieldControl(valueParam);
	}

	public static Control createSuitable(Param param) {

		if (param instanceof EnumParam) {
			return new EnumControl((EnumParam) param);
		}
        if (param instanceof BooleanParam) {
            return new CheckBoxControl((BooleanParam) param);
        }
        if (param instanceof DecimalParam) {
			DecimalParam decimalParam = (DecimalParam)param;
			if (param.getMin(Integer.class) != null && param.getMax(Integer.class) != null) {
				return new SliderControl(decimalParam);
			}
			return createComponentForText(decimalParam);
		}
		if (param instanceof FloatParam) {
			FloatParam floatParam = (FloatParam)param;
			if (param.getMin(Double.class) != null && param.getMax(Double.class) != null) {
				return new SliderControl(floatParam);
			}
			return createComponentForText(floatParam);
		}
		if (param instanceof StringParam) {
			return createComponentForText((StringParam)param);
		}
		if (param instanceof ProcedureParam) {
			return new ProcedureControl((ProcedureParam)param);
		}
		if (param instanceof BinaryParam) {
			return createComponentForText((BinaryParam)param);
		}
		if (param instanceof ColorParam) {
			return createComponentForText((ColorParam) param);
		}
		if (param instanceof UniversalParam) {
			return new TextAreaControl((UniversalParam)param);
		}
		if (param instanceof EventParam) {
			return new EventControl((EventParam)param);
		}
		return null;
	}

	protected void addAsOnlyChild(JComponent component) {
		this.setLayout(new BorderLayout());
		this.add(component, BorderLayout.CENTER);
	}

	@Override
	public String toString() {
		return param.toString();
	}
}
