package com.framsticks.gui.components;

import com.framsticks.params.types.EnumParam;
import com.framsticks.util.Numbers;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

@SuppressWarnings("serial")
public class EnumControl extends ValueControl {

	private JComboBox list;

	private static final Logger LOGGER = Logger.getLogger(EnumControl.class.getName());

	/**
	 * Minimum value index of list.
	 * TODO is it a base number?
	 */
	private int min;

	public EnumControl(EnumParam enumParam) {
		super(enumParam);

		this.min = param.getMin(Integer.class);

		list = new JComboBox();
		for (String item : enumParam.getElements()) {
			list.addItem(item);
		}
		list.setEditable(false);
		list.setEnabled(!isReadOnly());
		list.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent itemEvent) {
				notifyOfChange();
			}
		});
		addAsOnlyChild(list);

	}

	@Override
	public void setValueImpl(Object value) {
		Integer x = Numbers.cast(value, Integer.class);
		if (x == null) {
			return;
		}
		try {
			list.setSelectedIndex(x - min);
		} catch (IllegalArgumentException e) {
			LOGGER.error("failed to set value");
		}
	}

	@Override
	public Object getValue() {
		return list.getSelectedIndex();
	}

}
