package com.framsticks.gui.components;

import com.framsticks.params.CastFailure;
import com.framsticks.params.Flags;
import com.framsticks.params.types.ValueParam;

import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 * @author Piotr Sniegowski
 */
public abstract class ValueControl extends Control {

	protected ValueControlListener listener;

	public ValueControl(ValueParam valueParamEntry) {
		super(valueParamEntry);
	}

	protected abstract void setValueImpl(Object value);

	/** I consider this an ugly solution, but I was unable to find proper
	 * action listeners for underlying swing controls, that would only fire
	 * on user change and on programmatic change.
	 */
	protected boolean programmaticChange = false;

	public void setValue(Object value) {
		programmaticChange = true;
		setValueImpl(value);
		programmaticChange = false;
	}

	public abstract Object getValue();

	public void setListener(ValueControlListener listener) {
		this.listener = listener;
	}

	protected boolean notifyOfChange() {
		Object c = getValue();
		try {
			Object v = param.reassign(c, null);
			if (!programmaticChange) {
				return listener == null || listener.onChange(v);
			}
			return true;
		} catch (CastFailure castFailure) {
			return false;
		}
	}

	public boolean isReadOnly() {
		return (param.getFlags() & Flags.READONLY) != 0;
	}

    @Override
    public ValueParam getParam() {
        assert param instanceof ValueParam;
        return (ValueParam)param;
    }

	protected static DocumentListener createDocumentListener(final ValueControl component) {
		return new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent documentEvent) {
				component.notifyOfChange();
			}

			@Override
			public void removeUpdate(DocumentEvent documentEvent) {
				component.notifyOfChange();
			}

			@Override
			public void changedUpdate(DocumentEvent documentEvent) {
				component.notifyOfChange();
			}
		};
	}

}
