package com.framsticks.gui.view;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import java.util.Iterator;
import java.util.Vector;

/**
 * Table model for manages column in JTable on ListPanel.
 */
@SuppressWarnings("serial")
public class TableModel extends DefaultTableModel {

	/**
	 * List of columns.
	 */
	Vector<TableColumn> columns = new Vector<TableColumn>();

	/**
	 * Adds column to list.
	 *
	 * @param column TableColumn with getName, getId and index.
	 */
	public void addColumn(TableColumn column) {
		Iterator<TableColumn> iterator = columns.iterator();
		int index = 0;
		boolean inserted = false;
		while (iterator.hasNext()) {
			if (iterator.next().getModelIndex() < column.getModelIndex()) {
				index++;
			} else {
				columns.insertElementAt(column, index);
				inserted = true;
				break;
			}
		}
		if (!inserted) {
			columns.add(column);
		}
		setIdentifiers();
	}

	/**
	 * Removes column from list.
	 *
	 * @param identifier Column identifier.
	 */
	public void removeColumn(String identifier) {
		Iterator<TableColumn> iterator = columns.iterator();
		while (iterator.hasNext()) {
			if (iterator.next().getIdentifier().toString().equals(identifier)) {
				iterator.remove();
				break;
			}
		}
		setIdentifiers();
	}

	/**
	 * Returns list of columns.
	 *
	 * @return List of columns.
	 */
	public Vector<String> getColumns() {
		Vector<String> columnsId = new Vector<String>();
		for (TableColumn column : columns) {
			columnsId.add(column.getIdentifier().toString());
		}
		return columnsId;
	}

	/**
	 * Sets this model column identifiers.
	 */
	private void setIdentifiers() {
		Vector<String> newIdentifiers = new Vector<String>();
		for (TableColumn column : columns) {
			newIdentifiers.add(column.getHeaderValue().toString());
		}
		setColumnIdentifiers(newIdentifiers);
	}

	@Override
	public boolean isCellEditable(int arg0, int arg1) {
		return false;
	}

}
