package com.framsticks.hosting;

import com.framsticks.communication.*;
import com.framsticks.communication.queries.ApplicationRequest;
import com.framsticks.communication.queries.GetRequest;
import com.framsticks.communication.queries.InfoRequest;
import com.framsticks.core.Path;
import com.framsticks.params.*;
import com.framsticks.parsers.Savers;
import com.framsticks.core.LocalInstance;
import com.framsticks.util.Future;

import java.net.Socket;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * @author Piotr Sniegowski
 */
public class InstanceClient implements RequestHandler {

    protected final LocalInstance instance;
    protected final ServerConnection connection;

    public InstanceClient(LocalInstance instance, Socket socket) {
        this.instance = instance;
        connection = new ServerConnection(socket, this);
        connection.start();
    }

    @Override
    public String toString() {
        return instance + "|" + connection.toString();
    }


    @Override
    public void handle(final ApplicationRequest request, final ResponseCallback responseCallback) {
        instance.invokeLater(new Runnable() {
            @Override
            public void run() {
                final Path path = new Path(instance, request.getPath());
                if (!path.isResolved(request.getPath())) {
                    responseCallback.process(new Response(false, "\"invalid path\"", null));
                    return;
                }
                if (request instanceof GetRequest) {
                    instance.findInfo(path, new Future<FramsClass>() {
                        @Override
                        public void result(FramsClass result, Exception e) {
                            if (result == null) {
                                responseCallback.process(new Response(false, "\"failed to find info for access bind\"", null));
                                return;
                            }
                            List<File> files = new LinkedList<File>();
                            AccessInterface access = instance.bindAccess(path);

                            if (access == null) {

                                responseCallback.process(new Response(false, "\"failed to bind access\"", null));
                                return;
                            }
                            ListSink sink = new ListSink();
                            access.save(sink);
                            files.add(new File(path.getTextual(), new ListSource(sink.getOut())));
                            responseCallback.process(new Response(true, "", files));
                        }
                    });
                    return;
                }
                if (request instanceof InfoRequest) {
                    instance.findInfo(path, new Future<FramsClass>() {
                        @Override
                        public void result(FramsClass result, Exception e) {
                            if (result == null) {
                                responseCallback.process(new Response(false, "\"info not found\"", null));
                                return;
                            }
                            ListSink sink = new ListSink();
                            Savers.saveFramsClass(sink, result);
                            List<File> files = new ArrayList<File>();
                            files.add(new File(path.getTextual(), new ListSource(sink.getOut())));
                            responseCallback.process(new Response(true, null, files));
                        }
                    });
                    return;
                }
                responseCallback.process(new Response(false, "invalid", null));
            }
        });

    }
}
