package com.framsticks.params.types;

import com.framsticks.params.AccessInterface;
import com.framsticks.params.ArrayListAccess;
import com.framsticks.params.CastFailure;
import com.framsticks.util.Casting;
import com.framsticks.util.Containers;
import com.framsticks.util.Numbers;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Piotr Sniegowski
 */
public class ArrayListParam extends ListParam {

    public ArrayListParam(String containedTypeName) {
        super(containedTypeName);
    }


    @Override
    public Class getStorageType() {
        return List.class;
    }

    @Override
    public AccessInterface prepareAccessInterface(AccessInterface access) {
        return new ArrayListAccess(access);
    }

    @Override
    public String computeAccessId() {
        return "l " + containedTypeName;
    }

    @Override
    public Object reassign(Object newValue, Object oldValue) throws CastFailure {
        if (newValue == null) {
            throw new CastFailure();
        }
        Integer size = Numbers.cast(newValue, Integer.class);
        if (size != null) {
            //return oldValue;
            List list;
            if (oldValue == null) {
                list = new ArrayList();
            } else {
                list = Casting.tryCast(List.class, oldValue);
                if (list == null) {
                    throw new CastFailure();
                }
            }
            Containers.resizeList(list, size);
            return list;
        }
        if (oldValue != null) {
            return oldValue;
        }
        return newValue;
    }

    @Override
    public String getType() {
        return "l " + getContainedTypeName();
    }
}
