package com.framsticks.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Piotr Sniegowski
 */
public class ProgramArguments {

	private final Map<String, List<String>> arguments = new HashMap<String, List<String>>();

	public ProgramArguments(String[] args) {
		int a = 0;
		while (a < args.length) {
			if (args[a].startsWith("--")) {
				String argumentName = args[a].substring(2);

				String argumentValue;
				if (a + 1 < args.length && (!args[a + 1].startsWith("--"))) {
					argumentValue = args[a + 1];
					a += 2;
				} else {
					argumentValue = null;
					a += 1;
				}

				List<String> argumentValues;
				if (arguments.containsKey(argumentName)) {
					argumentValues = arguments.get(argumentName);
				} else {
					argumentValues = new ArrayList<String>();
					arguments.put(argumentName, argumentValues);
				}
				argumentValues.add(argumentValue);
			}
		}

	}

	public void forEach(String argument, UnaryFunctor<Boolean, String> functor) {
		if (!arguments.containsKey(argument)) {
			return;
		}
		for (String v : arguments.get(argument)) {
			if (!functor.call(v)) {
				break;
			}
		}
	}

	public String getSingle(String argument, String defValue) {
		if (!arguments.containsKey(argument)) {
			return defValue;
		}
		List<String> values = arguments.get(argument);
		if (values.size() != 1) {
			return defValue;
		}
		return values.get(0);
	}

}
